#pragma once

// std c++ headers
#include <type_traits>

namespace AMDiS
{
  namespace Traits
  {
    template <class T>
    struct IsIntegral
        : public std::is_integral<std::decay_t<T>> {};

    template <class T>
    struct IsArithmetic
        : public std::is_arithmetic<std::decay_t<T>> {};

  } // end namespace traits

  namespace Concepts
  {
    /** \addtogroup Concepts
     *  @{
     **/

    /// \brief The types following the std type-trait \ref std::is_integral are
    /// categorized as *integral types*.
    template <class T>
    constexpr bool Integral = Traits::IsIntegral<T>::value;

    /// \brief The types following the std type-trait \ref std::is_arithmetic are
    /// categorized as *arithmetic types*.
    template <class T>
    constexpr bool Arithmetic = Traits::IsArithmetic<T>::value;

    /** @} **/

  } // end namespace Concepts

} // end namespace AMDiS
